<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/29/16
 * Time: 8:03 AM
 */

namespace qfq;

require_once(__DIR__ . '/../Constants.php');
require_once(__DIR__ . '/../store/Store.php');
require_once(__DIR__ . '/../store/T3Info.php');
require_once(__DIR__ . '/../store/Store.php');
require_once(__DIR__ . '/../report/Link.php');
require_once(__DIR__ . '/../database/Database.php');
require_once(__DIR__ . '/UserFormException.php');
require_once(__DIR__ . '/../helper/OnArray.php');


class AbstractException extends \Exception {

    public $messageArray = array();
    public $messageArrayDebug = array();
    public $store = null;

    protected $file = '';
    protected $line = '';

    /**
     * @return string
     */
    public function formatException() {
        $t3Vars = array();
        $arrShow = $this->messageArray;
        $htmlDebug = '';

        try {
            // In a very early stage, it might be possible that Store can't be initialized: take care not to use it.
            $store = Store::getInstance();
            $t3Vars = $store->getStore(STORE_TYPO3);
        } catch (UserFormException $e) {
            $store = null;
        } catch (CodeException $e) {
            $store = null;
        }

        if (empty($t3Vars)) {
            $t3Vars = T3Info::getVars();
        }

        $arrShow[EXCEPTION_TIMESTAMP] = date('Y.m.d H:i:s O');
        $arrShow[EXCEPTION_CODE] = $this->getCode();
        $arrShow[EXCEPTION_MESSAGE] = $this->getMessage();

        foreach ([EXCEPTION_FORM, EXCEPTION_FORM_ELEMENT, EXCEPTION_FORM_ELEMENT_COLUMN] as $key) {
            if (isset($arrShow[$key])) {
                if ($arrShow[$key] == '') {
                    unset($arrShow[$key]);
                }
            }
        }

//        $html .= "Code: " . htmlspecialchars($this->messageArray['Code']) . "<br>";
//        $html .= "Message: " . Support::wrapTag("<strong>", htmlspecialchars($this->messageArray['Message'])) . "<br>";

        // Debug Information
        if ($store !== null && Support::findInSet(SYSTEM_SHOW_DEBUG_INFO_YES, $store->getVar(SYSTEM_SHOW_DEBUG_INFO, STORE_SYSTEM))) {

            $arrDebugShow = array_merge([EXCEPTION_REPORT_FULL_LEVEL => Store::getVar(SYSTEM_REPORT_FULL_LEVEL, STORE_SYSTEM)], $this->messageArrayDebug);

            $arrDebugShow[EXCEPTION_SIP] = $store->getStore(STORE_SIP);
            $arrDebugShow[EXCEPTION_PAGE_ID] = $t3Vars[TYPO3_PAGE_ID];
            $arrDebugShow[EXCEPTION_TT_CONTENT_UID] = $t3Vars[TYPO3_TT_CONTENT_UID];

            $arrDebugHidden = array();
            $arrDebugHidden[EXCEPTION_FILE] = $this->getFile();
            $arrDebugHidden[EXCEPTION_LINE] = $this->getLine();
            $arrDebugHidden[EXCEPTION_STACKTRACE] = $this->getTraceAsString();

            // Optional existing arrays will be flatten.
            $arrDebugShow = OnArray::varExportArray($arrDebugShow);
            $arrDebugHidden = OnArray::varExportArray($arrDebugHidden);

            // Sanitize any HTML content.
            $arrDebugShow = OnArray::htmlentitiesOnArray($arrDebugShow);
            $arrDebugHidden = OnArray::htmlentitiesOnArray($arrDebugHidden);

            // In case there is a 'form' name given in SIP, we probably have a problem in a form and a direct link to
            // edit the broken form will be helpful.
            $storeSystem = $store->getStore(STORE_SYSTEM);
            if (!empty($storeSystem[SYSTEM_FORM])) {
                $arrDebugShow[EXCEPTION_EDIT_FORM] = $this->buildFormLink($storeSystem);
            }

            $htmlDebug = OnArray::arrayToHtmlTable($arrDebugShow, 'Debug', EXCEPTION_TABLE_CLASS);
            $hidden = OnArray::arrayToHtmlTable($arrDebugHidden, 'Details', EXCEPTION_TABLE_CLASS);

//            $hidden = 'mein test<br>und mehr content';
            // Show / hide with just CSS: http://jsfiddle.net/t5Nf8/1/
            $htmlDebug .= "<style>input[type=checkbox]:checked + label + table { display: none; }</style>" .
                "<input type='checkbox' checked id='stacktrace'><label for='stacktrace'>&nbsp;Show/hide more details</label>$hidden";
        }

        // Sanitize any HTML content.
        $arrShow = OnArray::htmlentitiesOnArray($arrShow);
        $html = OnArray::arrayToHtmlTable($arrShow, 'Error', EXCEPTION_TABLE_CLASS);
//        foreach ($show as $key => $value) {
//            $html .= "$key: " . Support::wrapTag("<strong>", $value) . "<br>";
//        }

        return $html . $htmlDebug;
    }


    /**
     * Build a FormEditor link to the broken form.
     */
    private function buildFormLink($storeSystem) {

        $linkForm = '';
        $linkFormElement = '';
        try {
            $db = new Database();
            $sql = "SELECT id FROM Form WHERE name='" . $storeSystem[SYSTEM_FORM] . "'";

            $r = $db->sql($sql, ROW_EXPECT_0_1);
            if (!is_numeric($r[F_ID])) {
                return '';
            }

            $sip = new Sip();
            $link = new Link($sip);

            // Link to 'Form'
            $linkForm = $link->renderLink(TOKEN_SIP . '|' . TOKEN_BOOTSTRAP_BUTTON . '|' . TOKEN_PAGE . ':' .
                $storeSystem[SYSTEM_EDIT_FORM_PAGE] . '&' . CLIENT_FORM . '=' . FORM_NAME_FORM . '&' .
                CLIENT_RECORD_ID . '=' . $r[F_ID] . '|' . TOKEN_TEXT . ':' . $storeSystem[SYSTEM_FORM]);

            // Link to 'FormElement'
            if (!empty($storeSystem[SYSTEM_FORM_ELEMENT_ID])) {
                $linkFormElement = $link->renderLink(TOKEN_SIP . '|' . TOKEN_BOOTSTRAP_BUTTON . '|' . TOKEN_PAGE .
                    ':' . $storeSystem[SYSTEM_EDIT_FORM_PAGE] . '&' . CLIENT_FORM . '=' . FORM_NAME_FORM_ELEMENT . '&' .
                    CLIENT_RECORD_ID . '=' . $storeSystem[SYSTEM_FORM_ELEMENT_ID] . '|' .
                    TOKEN_TEXT . ':' . $storeSystem[SYSTEM_FORM_ELEMENT_ID]);
            }

        } catch (\exception $e) {
        }

        return 'Form: ' . $linkForm . '&nbsp;&nbsp;  FormElement: ' . $linkFormElement;
    }
}